/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 6912560
 * @run main/othervm Bug6912560
 * @summary Make sure that file path canonicalization in
 * sun.util.calendar.ZoneInfoFile works with the default security
 * manager.
 */

import java.io.File;
import java.util.*;

public class Bug6912560 {
    public static void main(String[] args) {
        // set the user.timezone property
        String tzname = "Asia/Tokyo";
        System.setProperty("user.timezone", tzname);
        // make sure the timezone will be initialized by
        // the next call to TimeZone.getDefault()
        TimeZone.setDefault(null);

        System.setSecurityManager(new SecurityManager());
        TimeZone tz = TimeZone.getDefault();
        if (!tzname.equals(tz.getID())) {
            throw new RuntimeException("got " + tz.getID()
                                       + ", expected " + tzname);
        }
    }
}
